"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var Lint = require("tslint");
var tsutils = require("tsutils");
var util_1 = require("../support/util");
var Rule = (function (_super) {
    tslib_1.__extends(Rule, _super);
    function Rule() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Rule.prototype.applyWithProgram = function (sourceFile, program) {
        return this.applyWithWalker(new Walker(sourceFile, this.getOptions(), program));
    };
    Rule.metadata = {
        description: "Enforces the use of Finnish notation.",
        options: {
            properties: {
                functions: { type: "boolean" },
                methods: { type: "boolean" },
                parameters: { type: "boolean" },
                properties: { type: "boolean" },
                variables: { type: "boolean" }
            },
            type: "object"
        },
        optionsDescription: Lint.Utils.dedent(templateObject_1 || (templateObject_1 = tslib_1.__makeTemplateObject(["\n            An optional object with optional `functions`, `methods`, `parameters`,\n            `properties` and `variables` properties.\n            The properies are booleans and determine whether or not Finnish notation is enforced.\n            All properties default to `true`.\n            "], ["\n            An optional object with optional \\`functions\\`, \\`methods\\`, \\`parameters\\`,\n            \\`properties\\` and \\`variables\\` properties.\n            The properies are booleans and determine whether or not Finnish notation is enforced.\n            All properties default to \\`true\\`.\n            "]))),
        requiresTypeInfo: true,
        ruleName: "rxjs-finnish",
        type: "style",
        typescriptOnly: true
    };
    return Rule;
}(Lint.Rules.TypedRule));
exports.Rule = Rule;
var Walker = (function (_super) {
    tslib_1.__extends(Walker, _super);
    function Walker(sourceFile, rawOptions, program) {
        var _this = _super.call(this, sourceFile, rawOptions, program) || this;
        _this.validate = {
            functions: true,
            methods: true,
            parameters: true,
            properties: true,
            variables: true
        };
        var options = _this.getOptions()[0];
        if (options) {
            _this.validate = tslib_1.__assign({}, _this.validate, options);
        }
        return _this;
    }
    Walker.prototype.visitFunctionDeclaration = function (node) {
        if (this.validate.functions) {
            this.validateNode(node, node.type);
        }
        _super.prototype.visitFunctionDeclaration.call(this, node);
    };
    Walker.prototype.visitFunctionExpression = function (node) {
        if (this.validate.functions) {
            this.validateNode(node, node.type);
        }
        _super.prototype.visitFunctionExpression.call(this, node);
    };
    Walker.prototype.visitGetAccessor = function (node) {
        if (this.validate.properties) {
            this.validateNode(node);
        }
        _super.prototype.visitGetAccessor.call(this, node);
    };
    Walker.prototype.visitMethodDeclaration = function (node) {
        if (this.validate.methods) {
            this.validateNode(node, node.type);
        }
        _super.prototype.visitMethodDeclaration.call(this, node);
    };
    Walker.prototype.visitMethodSignature = function (node) {
        if (this.validate.methods) {
            this.validateNode(node, node.type);
        }
        _super.prototype.visitMethodSignature.call(this, node);
    };
    Walker.prototype.visitObjectLiteralExpression = function (node) {
        var _this = this;
        if (this.validate.properties) {
            node.properties.forEach(function (property) {
                if (property.name && !tsutils.isComputedPropertyName(property.name)) {
                    _this.validateNode(property);
                }
            });
        }
        _super.prototype.visitObjectLiteralExpression.call(this, node);
    };
    Walker.prototype.visitParameterDeclaration = function (node) {
        if (this.validate.parameters) {
            this.validateNode(node);
        }
        _super.prototype.visitParameterDeclaration.call(this, node);
    };
    Walker.prototype.visitPropertyDeclaration = function (node) {
        if (this.validate.properties) {
            this.validateNode(node);
        }
        _super.prototype.visitPropertyDeclaration.call(this, node);
    };
    Walker.prototype.visitPropertySignature = function (node) {
        if (this.validate.properties) {
            this.validateNode(node);
        }
        _super.prototype.visitPropertySignature.call(this, node);
    };
    Walker.prototype.visitSetAccessor = function (node) {
        if (this.validate.properties) {
            this.validateNode(node);
        }
        _super.prototype.visitSetAccessor.call(this, node);
    };
    Walker.prototype.visitVariableDeclarationList = function (node) {
        var _this = this;
        if (this.validate.variables) {
            tsutils.forEachDeclaredVariable(node, function (variable) {
                _this.validateNode(variable);
            });
        }
        _super.prototype.visitVariableDeclarationList.call(this, node);
    };
    Walker.prototype.validateNode = function (node, typeNode) {
        var name = node.name;
        if (name) {
            var text = name.getText();
            var type = this.getTypeChecker().getTypeAtLocation(typeNode || node);
            if (!/\$$/.test(text) && util_1.couldBeType(type, "Observable")) {
                this.addFailureAtNode(node, "Finnish notation required for " + text);
            }
        }
    };
    return Walker;
}(Lint.ProgramAwareRuleWalker));
var templateObject_1;
