"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var Lint = require("tslint");
var tsutils = require("tsutils");
var util_1 = require("../support/util");
var Rule = (function (_super) {
    tslib_1.__extends(Rule, _super);
    function Rule() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Rule.prototype.applyWithProgram = function (sourceFile, program) {
        return this.applyWithWalker(new Walker(sourceFile, this.getOptions(), program));
    };
    Rule.metadata = {
        description: "Disallows calling the unsubscribe method of a subject instance.",
        options: null,
        optionsDescription: "Not configurable.",
        requiresTypeInfo: true,
        ruleName: "rxjs-no-subject-unsubscribe",
        type: "functionality",
        typescriptOnly: true
    };
    Rule.FAILURE_STRING = "Calling unsubscribe on a subject is forbidden";
    return Rule;
}(Lint.Rules.TypedRule));
exports.Rule = Rule;
var Walker = (function (_super) {
    tslib_1.__extends(Walker, _super);
    function Walker() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Walker.prototype.visitCallExpression = function (node) {
        var _this = this;
        node.forEachChild(function (child) {
            if (tsutils.isPropertyAccessExpression(child)) {
                var name_1 = child.name.getText();
                var typeChecker = _this.getTypeChecker();
                var type = typeChecker.getTypeAtLocation(child.expression);
                if ((name_1 === "unsubscribe") && util_1.isReferenceType(type) && util_1.couldBeType(type.target, "Subject")) {
                    _this.addFailureAtNode(child.name, Rule.FAILURE_STRING);
                }
                else if ((name_1 === "add") && util_1.isReferenceType(type) && util_1.couldBeType(type.target, "Subscription")) {
                    var argument = node.arguments[0];
                    var argumentType = typeChecker.getTypeAtLocation(argument);
                    if (util_1.couldBeType(argumentType, "Subject")) {
                        _this.addFailureAtNode(child.name, Rule.FAILURE_STRING);
                    }
                }
            }
        });
        _super.prototype.visitCallExpression.call(this, node);
    };
    return Walker;
}(Lint.ProgramAwareRuleWalker));
exports.Walker = Walker;
